import json
import logging
import sys

from .booking import BookingManager

logging.basicConfig(level=logging.INFO)


def main():
    config_path = sys.argv[1]
    with open(config_path) as f:
        users = json.load(f)
    for username, password in users.items():
        logging.info(f"[{username}] Booking seat...")
        try:
            manager = BookingManager(username, password)
            reservation_id = manager.book()
            logging.info(f"[{username}] Reservation id: {reservation_id}"
                         if reservation_id else f"[{username}] No reservation id found. Ignored.")
        except Exception:
            logging.exception(f"[{username}] Exception occur.")
    logging.info("Done.")


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import re
from os import path

from setuptools import find_packages, setup

curpath = path.abspath(path.dirname(__file__))
with open(path.join(curpath, "README.md")) as f:
    long_description = f.read()


def read(*parts):
    with open(path.join(curpath, *parts), 'r') as fp:
        return fp.read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


setup(
    name='sjtu_lib_booking',
    version=find_version("sjtu_lib_booking", "__init__.py"),
    description='A simple script that helps you book a seat in library.',

    author='LightQuantum',
    author_email='self@lightquantum.me',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=find_packages(),
    url='https://github.com/PhotonQuantum/sjtu-lib-booking',
    classifiers=(
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3 :: Only',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Environment :: Web Environment',
        'Intended Audience :: End Users/Desktop',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content'
    ),
    install_requires=[
        'pysjtu[ocr] >= 0.3.2'
    ],
    entry_points={
        'console_scripts': [
            "sjtu-lib-booking=sjtu_lib_booking.__main__:main"
        ]
    }
)



import re
from datetime import date, datetime
from typing import List, Optional

from httpx.exceptions import RedirectLoop
from lxml import html
from pysjtu import Client, Session

from .models import Reservation

HEADERS = {'Content-Type': 'application/x-www-form-urlencoded',
           'Host': 'booking.lib.sjtu.edu.cn',
           'Origin': 'http://booking.lib.sjtu.edu.cn',
           'Referer': 'http://booking.lib.sjtu.edu.cn/zg.asp',
           'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64; rv:79.0) Gecko/20100101 Firefox/79.0'}
sn_pattern = re.compile("预约号:(?P<sn>.*)，可至")


class BookingManager:
    def __init__(self, username: str, password: str):
        self.session = Session(username, password)
        self.client = Client(self.session)

    def book(self, book_date: Optional[date] = None) -> Optional[int]:
        if not book_date:
            book_date = datetime.now().date()
        profile = self.client.profile
        payload = {'yuyuechangguan': '主馆',
                   'user_name': profile.name,
                   'xgh': profile.student_id,
                   'mobile': profile.cellphone,
                   'yuyueshijian': book_date.strftime("%Y-%m-%d"),
                   'chengnuo': 'on'}
        while True:
            try:
                self.session.get("http://booking.lib.sjtu.edu.cn/index.asp")
                self.session.get("http://booking.lib.sjtu.edu.cn/zg.asp")
                r = self.session.post("http://booking.lib.sjtu.edu.cn/submit_zg.asp", headers=HEADERS, data=payload)
                break
            except RedirectLoop:
                pass
        sn = int(result.group("sn")) if (result := sn_pattern.search(r.text)) else None
        return sn

    def reservations(self) -> List[Reservation]:
        def tr_to_reservation(el: html.HtmlElement) -> Optional[Reservation]:
            tds: List[html.HtmlElement] = el.xpath(".//td")
            if len(tds) != 4:
                return None
            if tds[0].keys():
                return None
            return Reservation(int(tds[0].text), datetime.strptime(tds[2].text, "%Y-%m-%d").date())

        while True:
            try:
                r = self.session.get("http://booking.lib.sjtu.edu.cn/my2.asp")
                break
            except RedirectLoop:
                pass
        parsed_html = html.fromstring(r.text)
        trs = parsed_html.xpath("/html/body/main/section/div/div[2]/table[1]/tbody/tr")
        return list(filter(lambda x: x is not None, map(tr_to_reservation, trs)))
